@@ -1,16 +1,22 @@
 import os; from os.path import join
 import shutil
 from unittest import TestCase
-from tests import registerCase
-from mutagen.id3 import ID3, BitPaddedInt, COMR, Frames, Frames_2_2
+from tests import add
+from mutagen.id3 import ID3, BitPaddedInt, COMR, Frames, Frames_2_2, ID3Warning
+from StringIO import StringIO
+import warnings
+warnings.simplefilter('error', ID3Warning)
 
 try: from sets import Set as set
 except ImportError: pass
 
+_22 = ID3(); _22.version = (2,2,0)
 _23 = ID3(); _23.version = (2,3,0)
 _24 = ID3(); _24.version = (2,4,0)
 
 class ID3GetSetDel(TestCase):
+    uses_mmap = False
+
     def setUp(self):
         self.i = ID3()
         self.i["BLAH"] = 1
@@ -60,6 +66,8 @@ class ID3GetSetDel(TestCase):
         self.assert_(self.i.getall("FOOB") in [[t, t2], [t2, t]])
 
 class ID3Loading(TestCase):
+    uses_mmap = False
+
 
     empty = join('tests', 'data', 'emptyfile.mp3')
     silence = join('tests', 'data', 'silence-44-s.mp3')
@@ -80,16 +88,71 @@ class ID3Loading(TestCase):
     def test_header_empty(self):
         id3 = ID3()
         id3._ID3__fileobj = file(self.empty, 'rb')
-        self.assertRaises(EOFError, id3.load_header)
+        self.assertRaises(EOFError, id3._ID3__load_header)
 
     def test_header_silence(self):
         id3 = ID3()
         id3._ID3__fileobj = file(self.silence, 'rb')
-        id3.load_header()
+        id3._ID3__load_header()
         self.assertEquals(id3.version, (2,3,0))
-        self.assertEquals(id3._size, 1304)
+        self.assertEquals(id3.size, 1314)
+
+    def test_header_2_4_invalid_flags(self):
+        id3 = ID3()
+        id3._ID3__fileobj = StringIO('ID3\x04\x00\x1f\x00\x00\x00\x00')
+        self.assertRaises(ValueError, id3._ID3__load_header)
+
+    def test_header_2_3_invalid_flags(self):
+        id3 = ID3()
+        id3._ID3__fileobj = StringIO('ID3\x03\x00\x1f\x00\x00\x00\x00')
+        self.assertRaises(ValueError, id3._ID3__load_header)
+
+    def test_header_2_2(self):
+        id3 = ID3()
+        id3._ID3__fileobj = StringIO('ID3\x02\x00\x00\x00\x00\x00\x00')
+        id3._ID3__load_header()
+        self.assertEquals(id3.version, (2,2,0))
+
+    def test_header_2_1(self):
+        id3 = ID3()
+        id3._ID3__fileobj = StringIO('ID3\x01\x00\x00\x00\x00\x00\x00')
+        self.assertRaises(NotImplementedError, id3._ID3__load_header)
+
+    def test_header_too_small(self):
+        id3 = ID3()
+        id3._ID3__fileobj = StringIO('ID3\x01\x00\x00\x00\x00\x00')
+        self.assertRaises(EOFError, id3._ID3__load_header)
+
+    def test_header_2_4_extended(self):
+        id3 = ID3()
+        id3._ID3__fileobj = StringIO(
+            'ID3\x04\x00\x40\x00\x00\x00\x00\x00\x00\x00\x05\x5a')
+        id3._ID3__load_header()
+        self.assertEquals(id3._ID3__extsize, 5)
+        self.assertEquals(id3._ID3__extdata, '\x5a')
+
+    def test_unsynch(self):
+        id3 = ID3()
+        id3.version = (2,4,0)
+        id3._ID3__flags = 0x80
+        badsync = '\x00\xff\x00ab\x00'
+        self.assertEquals(
+            id3._ID3__load_framedata(Frames["TPE2"], 0, badsync), [u"\xffab"])
+        id3._ID3__flags = 0x00
+        self.assertEquals(id3._ID3__load_framedata(
+            Frames["TPE2"],0x40, badsync), [u"\xffab"])
+        tag = id3._ID3__load_framedata(Frames["TPE2"], 0, badsync)
+        self.assertEquals(tag, [u"\xff", u"ab"])
+
+    def test_insane__ID3__fullread(self):
+        id3 = ID3()
+        id3._ID3__filesize = 0
+        self.assertRaises(ValueError, id3._ID3__fullread, -3)
+        self.assertRaises(EOFError, id3._ID3__fullread, 3)
 
 class ID3Tags(TestCase):
+    uses_mmap = False
+
     def setUp(self):
         self.silence = join('tests', 'data', 'silence-44-s.mp3')
 
@@ -136,9 +199,42 @@ class ID3Tags(TestCase):
         self.assertEquals(2, +id3['TRCK'])
         self.assertEquals('2004', id3['TDRC'])
 
+    def test_badencoding(self):
+        self.assertRaises(IndexError, Frames["TPE1"].fromData, _24, 0, "\x09ab")
+        self.assertRaises(ValueError, Frames["TPE1"], encoding=9, text="ab")
+
+    def test_badsync(self):
+        self.assertRaises(
+            ValueError, Frames["TPE1"].fromData, _24, 0x02, "\x00\xff\xfe")
+
+    def test_noencrypt(self):
+        self.assertRaises(
+            NotImplementedError, Frames["TPE1"].fromData, _24, 0x04, "\x00")
+        self.assertRaises(
+            NotImplementedError, Frames["TPE1"].fromData, _23, 0x40, "\x00")
+
+    def test_badcompress(self):
+        self.assertRaises(
+            ValueError, Frames["TPE1"].fromData, _24, 0x08, "\x00\x00\x00\x00#")
+        self.assertRaises(
+            ValueError, Frames["TPE1"].fromData, _23, 0x80, "\x00\x00\x00\x00#")
+
+    def test_junkframe(self):
+        self.assertRaises(ValueError, Frames["TPE1"].fromData, _24, 0, "")
+
+    def test_extradata(self):
+        from mutagen.id3 import RVRB, RBUF
+        self.assertRaises(ID3Warning,
+                RVRB()._readData, 'L1R1BBFFFFPP#xyz')
+        self.assertRaises(ID3Warning,
+                RBUF()._readData, '\x00\x01\x00\x01\x00\x00\x00\x00#xyz')
+
 class ID3v1Tags(TestCase):
-    silence = join('tests', 'data', 'silence-44-s-v1.mp3')
-    id3 = ID3(silence)
+    uses_mmap = False
+
+    def setUp(self):
+        self.silence = join('tests', 'data', 'silence-44-s-v1.mp3')
+        self.id3 = ID3(self.silence)
 
     def test_album(self):
         self.assertEquals('Quod Libet Test Data', self.id3['TALB'])
@@ -154,6 +250,16 @@ class ID3v1Tags(TestCase):
     def test_year(self):
         self.assertEquals('2004', self.id3['TDRC'])
 
+    def test_v1_not_v11(self):
+        from mutagen.id3 import MakeID3v1, ParseID3v1, TRCK
+        self.id3["TRCK"] = TRCK(encoding=0, text="32")
+        tag = MakeID3v1(self.id3)
+        self.failUnless(32, ParseID3v1(tag)["TRCK"])
+        del(self.id3["TRCK"])
+        tag = MakeID3v1(self.id3)
+        tag = tag[:125] + '  ' + tag[-1]
+        self.failIf("TRCK" in ParseID3v1(tag))
+
     def test_nulls(self):
         from mutagen.id3 import ParseID3v1
         s = 'TAG%(title)30s%(artist)30s%(album)30s%(year)4s%(cmt)29s\x03\x01'
@@ -184,6 +290,90 @@ class ID3v1Tags(TestCase):
             frames[key] = self.id3[key]
         self.assertEquals(ParseID3v1(MakeID3v1(frames)), frames)
 
+    def test_make_from_empty(self):
+        from mutagen.id3 import MakeID3v1, TCON, COMM
+        empty = 'TAG' + '\x00' * 124 + '\xff'
+        self.assertEquals(MakeID3v1({}), empty)
+        self.assertEquals(MakeID3v1({'TCON': TCON()}), empty)
+        self.assertEquals(
+            MakeID3v1({'COMM': COMM(encoding=0, text="")}), empty)
+
+    def test_invalid(self):
+        from mutagen.id3 import ParseID3v1
+        self.failUnless(ParseID3v1("") is None)
+
+    def test_invalid_track(self):
+        from mutagen.id3 import ParseID3v1, MakeID3v1, TRCK
+        tag = {}
+        tag["TRCK"] = TRCK(encoding=0, text="not a number")
+        v1tag = MakeID3v1(tag)
+        self.failIf("TRCK" in ParseID3v1(v1tag))
+
+    def test_v1_genre(self):
+        from mutagen.id3 import ParseID3v1, MakeID3v1, TCON
+        tag = {}
+        tag["TCON"] = TCON(encoding=0, text="Pop")
+        v1tag = MakeID3v1(tag)
+        self.failUnlessEqual(ParseID3v1(v1tag)["TCON"].genres, ["Pop"])
+
+class TestWriteID3v1(TestCase):
+    SILENCE = os.path.join("tests", "data", "silence-44-s.mp3")
+    def setUp(self):
+        from tempfile import mkstemp
+        fd, self.filename = mkstemp(suffix='.mp3')
+        os.close(fd)
+        shutil.copy(self.SILENCE, self.filename)
+        self.audio = ID3(self.filename)
+
+    def failIfV1(self):
+        fileobj = file(self.filename, "rb")
+        fileobj.seek(-128, 2)
+        self.failIf(fileobj.read(3) == "TAG")
+
+    def failUnlessV1(self):
+        fileobj = file(self.filename, "rb")
+        fileobj.seek(-128, 2)
+        self.failUnless(fileobj.read(3) == "TAG")
+
+    def test_save_delete(self):
+        self.audio.save(v1=0)
+        self.failIfV1()
+
+    def test_save_add(self):
+        self.audio.save(v1=2)
+        self.failUnlessV1()
+
+    def test_save_add(self):
+        self.audio.save(v1=2)
+        self.failUnlessV1()
+
+    def test_save_defaults(self):
+        self.audio.save(v1=0)
+        self.failIfV1()
+        self.audio.save(v1=1)
+        self.failIfV1()
+        self.audio.save(v1=2)
+        self.failUnlessV1()
+        self.audio.save(v1=1)
+        self.failUnlessV1()
+
+    def tearDown(self):
+        os.unlink(self.filename)
+
+add(TestWriteID3v1)
+
+class TestV22Tags(TestCase):
+    uses_mmap = False
+
+    def setUp(self):
+        filename = os.path.join("tests", "data", "id3v22-test.mp3")
+        self.tags = ID3(filename)
+
+    def test_tags(self):
+        self.failUnless(self.tags["TRCK"].text == ["3/11"])
+        self.failUnless(self.tags["TPE1"].text == ["Anais Mitchell"])
+add(TestV22Tags)
+
 def TestReadTags():
     tests = [
     ['TALB', '\x00a/b', 'a/b', '', dict(encoding=0)],
@@ -193,9 +383,11 @@ def TestReadTags():
     ['TCOP', '\x001900 c', '1900 c', '', dict(encoding=0)],
     ['TDAT', '\x00a/b', 'a/b', '', dict(encoding=0)],
     ['TDEN', '\x001987', '1987', '', dict(encoding=0, year=[1987])],
-    ['TDOR', '\x001987-12', '1987-12', '', dict(encoding=0, year=[1987], month=[12])],
+    ['TDOR', '\x001987-12', '1987-12', '',
+     dict(encoding=0, year=[1987], month=[12])],
     ['TDRC', '\x001987\x00', '1987', '', dict(encoding=0, year=[1987])],
-    ['TDRL', '\x001987\x001988', '1987,1988', '', dict(encoding=0, year=[1987,1988])],
+    ['TDRL', '\x001987\x001988', '1987,1988', '',
+     dict(encoding=0, year=[1987,1988])],
     ['TDTG', '\x001987', '1987', '', dict(encoding=0, year=[1987])],
     ['TDLY', '\x001205', '1205', 1205, dict(encoding=0)],
     ['TENC', '\x00a b/c d', 'a b/c d', '', dict(encoding=0)],
@@ -239,7 +431,8 @@ def TestReadTags():
     ['TSST', '\x0012345', '12345', '', dict(encoding=0)],
     ['TYER', '\x002004', '2004', 2004, dict(encoding=0)],
 
-    ['TXXX', '\x00usr\x00a/b\x00c', ['a/b','c'], '', dict(encoding=0, desc='usr')],
+    ['TXXX', '\x00usr\x00a/b\x00c', ['a/b','c'], '',
+     dict(encoding=0, desc='usr')],
 
     ['WCOM', 'http://foo', 'http://foo', '', {}],
     ['WCOP', 'http://bar', 'http://bar', '', {}],
@@ -257,22 +450,28 @@ def TestReadTags():
 
     ['MCDI', '\x01\x02\x03\x04', '\x01\x02\x03\x04', '', {}],
     
-    #'ETCO', '\x01\x12\x00\x00\x7f\xff', [(18,32767)], '', dict(format=1)],
+    ['ETCO', '\x01\x12\x00\x00\x7f\xff', [(18, 32767)], '', dict(format=1)],
 
     ['COMM', '\x00ENUT\x00Com', 'Com', '',
         dict(desc='T', lang='ENU', encoding=0)],
+    # found in a real MP3
+    ['COMM', '\x00\x00\xcc\x01\x00     ', '     ', '',
+        dict(desc=u'', lang='\x00\xcc\x01', encoding=0)],
+
     ['APIC', '\x00-->\x00\x03cover\x00cover.jpg', 'cover.jpg', '',
         dict(mime='-->', type=3, desc='cover', encoding=0)],
     ['USER', '\x00ENUCom', 'Com', '', dict(lang='ENU', encoding=0)],
 
     ['RVA2', 'testdata\x00\x01\xfb\x8c\x10\x12\x23',
-        'Master volume: -2.226562 dB/0.141693', '',
-        dict(desc='testdata', channel=1,
-             # FIXME: we need to use almost
-             gain=-2.22656, peak=0.14169)],
+        'Master volume: -2.2266 dB/0.1417', '',
+        dict(desc='testdata', channel=1, gain=-2.22656, peak=0.14169)],
+
+    ['RVA2', 'testdata\x00\x01\xfb\x8c\x24\x01\x22\x30\x00\x00',
+        'Master volume: -2.2266 dB/0.1417', '',
+        dict(desc='testdata', channel=1, gain=-2.22656, peak=0.14169)],
 
     ['RVA2', 'testdata2\x00\x01\x04\x01\x00',
-        'Master volume: +2.001953 dB/0.000000', '',
+        'Master volume: +2.0020 dB/0.0000', '',
         dict(desc='testdata2', channel=1, gain=2.001953125, peak=0)],
 
     ['PCNT', '\x00\x00\x00\x11', 17, 17, dict(count=17)],
@@ -280,23 +479,34 @@ def TestReadTags():
         dict(email="foo@bar.org", rating=222, count=17)],
 
     ['UFID', 'own\x00data', 'data', '', dict(data='data', owner='own')],
+    ['UFID', 'own\x00\xdd', '\xdd', '', dict(data='\xdd', owner='own')],
+
     ['GEOB', '\x00mime\x00name\x00desc\x00data', 'data', '',
         dict(encoding=0, mime='mime', filename='name', desc='desc')],
 
     ['USLT', '\x00engsome lyrics\x00woo\nfun', 'woo\nfun', '',
      dict(encoding=0, lang='eng', desc='some lyrics', text='woo\nfun')],
 
+    ['SYLT', ('\x00eng\x02\x01some lyrics\x00foo\x00\x00\x00\x00\x01bar'
+              '\x00\x00\x00\x00\x10'), "foobar", '',
+     dict(encoding=0, lang='eng', type=1, format=2, desc='some lyrics')],
+
     ['POSS', '\x01\x0f', 15, 15, dict(format=1, position=15)],
     ['OWNE', '\x00USD10.01\x0020041010CDBaby', 'CDBaby', 'CDBaby',
      dict(encoding=0, price="USD10.01", date='20041010', seller='CDBaby')],
 
     ['PRIV', 'a@b.org\x00random data', 'random data', 'random data',
      dict(owner='a@b.org', data='random data')],
+    ['PRIV', 'a@b.org\x00\xdd', '\xdd', '\xdd',
+     dict(owner='a@b.org', data='\xdd')],
+
     ['SIGN', '\x92huh?', 'huh?', 'huh?', dict(group=0x92, sig='huh?')],
     ['ENCR', 'a@b.org\x00\x92Data!', 'Data!', 'Data!',
      dict(owner='a@b.org', method=0x92, data='Data!')],
     ['SEEK', '\x00\x12\x00\x56', 0x12*256*256+0x56, 0x12*256*256+0x56,
      dict(offset=0x12*256*256+0x56)],
+
+    ['SYTC', "\x01\x10obar", '\x10obar', '', dict(format=1, data='\x10obar')],
     
     ['RBUF', '\x00\x12\x00', 0x12*256, 0x12*256, dict(size=0x12*256)],
     ['RBUF', '\x00\x12\x00\x01', 0x12*256, 0x12*256,
@@ -335,8 +545,32 @@ def TestReadTags():
         encoding=0, price="USD10.00", valid_until="20051010",
         contact="ql@sc.net", format=9, seller="Joe", desc="A song")],
 
+    ['MLLT', '\x00\x01\x00\x00\x02\x00\x00\x03\x04\x08foobar', 'foobar', '',
+     dict(frames=1, bytes=2, milliseconds=3, bits_for_bytes=4,
+          bits_for_milliseconds=8, data='foobar')],
+
+    ['EQU2', '\x00Foobar\x00\x01\x01\x04\x00', [(128.5, 2.0)], '',
+     dict(method=0, desc="Foobar")],
+
+    ['ASPI', '\x00\x00\x00\x00\x00\x00\x00\x10\x00\x03\x08\x01\x02\x03',
+     [1, 2, 3], '', dict(S=0, L=16, N=3, b=8)],
+
+    ['ASPI', '\x00\x00\x00\x00\x00\x00\x00\x10\x00\x03\x10'
+     '\x00\x01\x00\x02\x00\x03', [1, 2, 3], '', dict(S=0, L=16, N=3, b=16)],
+
+    ['LINK', 'TIT1http://www.example.org/TIT1.txt\x00',
+     ("TIT1", 'http://www.example.org/TIT1.txt'), '',
+     dict(frameid='TIT1', url='http://www.example.org/TIT1.txt')],
+    ['LINK', 'COMMhttp://www.example.org/COMM.txt\x00engfoo',
+     ("COMM", 'http://www.example.org/COMM.txt', 'engfoo'), '',
+     dict(frameid='COMM', url='http://www.example.org/COMM.txt',
+          data='engfoo')],
+
     # 2.2 tags
     ['UFI', 'own\x00data', 'data', '', dict(data='data', owner='own')],
+    ['SLT', ('\x00eng\x02\x01some lyrics\x00foo\x00\x00\x00\x00\x01bar'
+              '\x00\x00\x00\x00\x10'), "foobar", '',
+     dict(encoding=0, lang='eng', type=1, format=2, desc='some lyrics')],
     ['TT1', '\x00ab\x00', 'ab', '', dict(encoding=0)],
     ['TT2', '\x00ab', 'ab', '', dict(encoding=0)],
     ['TT3', '\x00ab', 'ab', '', dict(encoding=0)],
@@ -386,10 +620,13 @@ def TestReadTags():
         dict(encoding=0)],
     ['MCI', '\x01\x02\x03\x04', '\x01\x02\x03\x04', '', {}],
 
+    ['ETC', '\x01\x12\x00\x00\x7f\xff', [(18, 32767)], '', dict(format=1)],
+
     ['COM', '\x00ENUT\x00Com', 'Com', '',
         dict(desc='T', lang='ENU', encoding=0)],
     ['PIC', '\x00-->\x03cover\x00cover.jpg', 'cover.jpg', '',
         dict(mime='-->', type=3, desc='cover', encoding=0)],
+
     ['POP', 'foo@bar.org\x00\xde\x00\x00\x00\x11', 222, 222,
         dict(email="foo@bar.org", rating=222, count=17)],
     ['CNT', '\x00\x00\x00\x11', 17, 17, dict(count=17)],
@@ -409,17 +646,27 @@ def TestReadTags():
      (0x12*256+0x12, 0x23*256+0x23), '',
      dict(left=0x12*256+0x12, right=0x23*256+0x23) ],
 
+    ['STC', "\x01\x10obar", '\x10obar', '', dict(format=1, data='\x10obar')],
+
+    ['MLL', '\x00\x01\x00\x00\x02\x00\x00\x03\x04\x08foobar', 'foobar', '',
+     dict(frames=1, bytes=2, milliseconds=3, bits_for_bytes=4,
+          bits_for_milliseconds=8, data='foobar')],
+    ['LNK', 'TT1http://www.example.org/TIT1.txt\x00',
+     ("TT1", 'http://www.example.org/TIT1.txt'), '',
+     dict(frameid='TT1', url='http://www.example.org/TIT1.txt')],
+    ['CRM', 'foo@example.org\x00test\x00woo',
+     'woo', '', dict(owner='foo@example.org', desc='test', data='woo')],
 
     ]
 
-
     load_tests = {}
     repr_tests = {}
     write_tests = {}
     for i, (tag, data, value, intval, info) in enumerate(tests):
         info = info.copy()
 
-        def test_tag(self, tag=tag, data=data, value=value, intval=intval, info=info):
+        def test_tag(self, tag=tag, data=data, value=value, intval=intval,
+                     info=info):
             from operator import pos
             id3 = __import__('mutagen.id3', globals(), locals(), [tag])
             TAG = getattr(id3, tag)
@@ -471,11 +718,14 @@ def TestReadTags():
         write_tests['test_write_%s_%d' % (tag, i)] = test_tag_write
 
     testcase = type('TestReadTags', (TestCase,), load_tests)
-    registerCase(testcase)
+    testcase.uses_mmap = False
+    add(testcase)
     testcase = type('TestReadReprTags', (TestCase,), repr_tests)
-    registerCase(testcase)
+    testcase.uses_mmap = False
+    add(testcase)
     testcase = type('TestReadWriteTags', (TestCase,), write_tests)
-    registerCase(testcase)
+    testcase.uses_mmap = False
+    add(testcase)
 
     test_tests = {}
     from mutagen.id3 import Frames, Frames_2_2
@@ -485,12 +735,14 @@ def TestReadTags():
         def check(self, tag=tag): self.assert_(tag in tested_tags)
         tested_tags['test_' + tag + '_tested'] = check
     testcase = type('TestTestedTags', (TestCase,), tested_tags)
-    registerCase(testcase)
+    testcase.uses_mmap = False
+    add(testcase)
 
 TestReadTags()
 del TestReadTags
 
 class BitPaddedIntTest(TestCase):
+    uses_mmap = False
 
     def test_zero(self):
         self.assertEquals(BitPaddedInt('\x00\x00\x00\x00'), 0)
@@ -547,6 +799,7 @@ class BitPaddedIntTest(TestCase):
         self.assertEquals(len(BitPaddedInt.to_str(2**32, width=-1)), 5)
 
 class SpecSanityChecks(TestCase):
+    uses_mmap = False
 
     def test_bytespec(self):
         from mutagen.id3 import ByteSpec
@@ -609,6 +862,7 @@ class SpecSanityChecks(TestCase):
         self.assertEquals('\xfc\x00', s.write(None, -2.0))
 
 class FrameSanityChecks(TestCase):
+    uses_mmap = False
 
     def test_TF(self):
         from mutagen.id3 import TextFrame
@@ -628,7 +882,8 @@ class FrameSanityChecks(TestCase):
 
     def test_NTPF(self):
         from mutagen.id3 import NumericPartTextFrame
-        self.assert_(isinstance(NumericPartTextFrame(text='1/2'), NumericPartTextFrame))
+        self.assert_(
+            isinstance(NumericPartTextFrame(text='1/2'), NumericPartTextFrame))
 
     def test_MTF(self):
         from mutagen.id3 import TextFrame
@@ -639,21 +894,37 @@ class FrameSanityChecks(TestCase):
         self.assert_(isinstance(TXXX(desc='d',text='text'), TXXX))
 
     def test_22_uses_direct_ints(self):
-        from mutagen.id3 import TT1, Frames_2_2
         data = 'TT1\x00\x00\x83\x00' + ('123456789abcdef' * 16)
         id3 = ID3()
         id3.version = (2,2,0)
-        tag = list(id3.read_frames(data, Frames_2_2))[0]
+        tag = list(id3._ID3__read_frames(data, Frames_2_2))[0]
         self.assertEquals(data[7:7+0x82].decode('latin1'), tag.text[0])
 
+    def test_frame_too_small(self):
+        self.assertEquals([], list(_24._ID3__read_frames('012345678', Frames)))
+        self.assertEquals([], list(_23._ID3__read_frames('012345678', Frames)))
+        self.assertEquals([], list(_22._ID3__read_frames('01234', Frames_2_2)))
+        self.assertEquals(
+            [], list(_22._ID3__read_frames('TT1'+'\x00'*3, Frames_2_2)))
+
+    def test_unknown_22_frame(self):
+        data = 'XYZ\x00\x00\x01\x00'
+        self.assertEquals([data], list(_22._ID3__read_frames(data, {})))
+
 
     def test_zlib_latin1(self):
         from mutagen.id3 import TPE1
-        tag = TPE1.fromData(_23, 0x80, '\x00\x00\x00\x0f'
+        tag = TPE1.fromData(_24, 0x9, '\x00\x00\x00\x0f'
                 'x\x9cc(\xc9\xc8,V\x00\xa2D\xfd\x92\xd4\xe2\x12\x00&\x7f\x05%')
         self.assertEquals(tag.encoding, 0)
         self.assertEquals(tag, ['this is a/test'])
 
+    def test_datalen_but_not_compressed(self):
+        from mutagen.id3 import TPE1
+        tag = TPE1.fromData(_24, 0x01, '\x00\x00\x00\x06\x00A test')
+        self.assertEquals(tag.encoding, 0)
+        self.assertEquals(tag, ['A test'])
+
     def test_utf8(self):
         from mutagen.id3 import TPE1
         tag = TPE1.fromData(_23, 0x00, '\x03this is a test')
@@ -662,8 +933,8 @@ class FrameSanityChecks(TestCase):
 
     def test_zlib_utf16(self):
         from mutagen.id3 import TPE1
-        data = '\x00\x00\x00\x1fx\x9cc\xfc\xff\xaf\x84!\x83!\x93\xa1\x98A' \
-                '\x01J&2\xe83\x940\xa4\x02\xd9%\x0c\x00\x87\xc6\x07#'
+        data = ('\x00\x00\x00\x1fx\x9cc\xfc\xff\xaf\x84!\x83!\x93\xa1\x98A'
+                '\x01J&2\xe83\x940\xa4\x02\xd9%\x0c\x00\x87\xc6\x07#')
         tag = TPE1.fromData(_23, 0x80, data)
         self.assertEquals(tag.encoding, 1)
         self.assertEquals(tag, ['this is a/test'])
@@ -688,10 +959,12 @@ class FrameSanityChecks(TestCase):
 
     def test_load_write(self):
         from mutagen.id3 import TPE1, Frames
-        artists= [s.decode('utf8') for s in ['\xc2\xb5', '\xe6\x97\xa5\xe6\x9c\xac']]
+        artists= [s.decode('utf8') for s in
+                  ['\xc2\xb5', '\xe6\x97\xa5\xe6\x9c\xac']]
         artist = TPE1(encoding=3, text=artists)
         id3 = ID3()
-        tag = list(id3.read_frames(id3.save_frame(artist), Frames))[0]
+        tag = list(id3._ID3__read_frames(
+            id3._ID3__save_frame(artist), Frames))[0]
         self.assertEquals('TPE1', type(tag).__name__)
         self.assertEquals(artist.text, tag.text)
 
@@ -724,7 +997,8 @@ class FrameSanityChecks(TestCase):
         from mutagen.id3 import COMM
         self.assertEquals(COMM(text="a").HashKey, COMM(text="b").HashKey)
         self.assertNotEquals(COMM(desc="a").HashKey, COMM(desc="b").HashKey)
-        self.assertNotEquals(COMM(lang="abc").HashKey, COMM(lang="def").HashKey)
+        self.assertNotEquals(
+            COMM(lang="abc").HashKey, COMM(lang="def").HashKey)
 
     def test_multi_RVA2(self):
         from mutagen.id3 import RVA2
@@ -754,51 +1028,69 @@ class FrameSanityChecks(TestCase):
     def test_multi_USER(self):
         from mutagen.id3 import USER
         self.assertEquals(USER(text="a").HashKey, USER(text="b").HashKey)
-        self.assertNotEquals(USER(lang="abc").HashKey, USER(lang="def").HashKey)
+        self.assertNotEquals(
+            USER(lang="abc").HashKey, USER(lang="def").HashKey)
 
-    def skip_test_harsh(self):
-        from os import walk
-        from traceback import print_exc
-        from mutagen.id3 import ID3NoHeaderError, ID3UnsupportedVersionError
-        total = 0
-        unsynch = 0
-        failures = {}
-        missing = []
-        for path, dirs, files in walk('/vault/music'):
-            for fn in files:
-                if not fn.lower().endswith('.mp3'): continue
-                ffn = join(path, fn)
-                try:
-                    total += 1
-                    if not total & 0xFF: print total
-                    #print ffn
-                    id3 = ID3()
-                    id3.PEDANTIC = False
-                    id3.load(ffn)
-                    for frame, val in id3.iteritems():
-                        pass #print frame, str(val)
-                except KeyboardInterrupt: raise
-                except ID3NoHeaderError:
-                    missing.append(ffn)
-                except ID3UnsupportedVersionError, err:
-                    failures.setdefault(err.__class__, []).append(fn)
-                except Exception, err:
-                    if err.__class__ not in failures:
-                        print_exc()
-                    failures.setdefault(err.__class__, []).append(ffn)
-                else:
-                    if id3.f_unsynch: unsynch += 1
+class UpdateTo24(TestCase):
+    uses_mmap = False
 
-        #for ffn in missing: print ffn
-        failcount = 0
-        for fail, files in failures.iteritems():
-            failcount += len(files)
-            print fail, len(files)
+    def test_pic(self):
+        from mutagen.id3 import PIC
+        id3 = ID3()
+        id3.version = (2, 2)
+        id3.add(PIC(encoding=0, mime="PNG", desc="cover", type=3, data=""))
+        id3.update_to_v24()
+        self.failUnlessEqual(id3["APIC:cover"].mime, "image/png")
 
-        total -= len(missing)
-        print total-failcount, '/', total, 'success [%d missing] (%d unsynch)' % (len(missing), unsynch)
+    def test_tyer(self):
+        from mutagen.id3 import TYER
+        id3 = ID3()
+        id3.version = (2, 3)
+        id3.add(TYER(encoding=0, text="2006"))
+        id3.update_to_v24()
+        self.failUnlessEqual(id3["TDRC"], "2006")
+
+    def test_tyer_tdat(self):
+        from mutagen.id3 import TYER, TDAT
+        id3 = ID3()
+        id3.version = (2, 3)
+        id3.add(TYER(encoding=0, text="2006"))
+        id3.add(TDAT(encoding=0, text="0603"))
+        id3.update_to_v24()
+        self.failUnlessEqual(id3["TDRC"], "2006-06-03")
+
+    def test_tyer_tdat_time(self):
+        from mutagen.id3 import TYER, TDAT, TIME
+        id3 = ID3()
+        id3.version = (2, 3)
+        id3.add(TYER(encoding=0, text="2006"))
+        id3.add(TDAT(encoding=0, text="0603"))
+        id3.add(TIME(encoding=0, text="1127"))
+        id3.update_to_v24()
+        self.failUnlessEqual(id3["TDRC"], "2006-06-03 11:27:00")
+
+    def test_tory(self):
+        from mutagen.id3 import TORY
+        id3 = ID3()
+        id3.version = (2, 3)
+        id3.add(TORY(encoding=0, text="2006"))
+        id3.update_to_v24()
+        self.failUnlessEqual(id3["TDOR"], "2006")
+
+    def test_ipls(self):
+        from mutagen.id3 import IPLS
+        id3 = ID3()
+        id3.version = (2, 3)
+        id3.add(IPLS(encoding=0, people=[["a", "b"], ["c", "d"]]))
+        id3.update_to_v24()
+        self.failUnlessEqual(id3["TIPL"], [["a", "b"], ["c", "d"]])
+
+
+add(UpdateTo24)
 
 class Genres(TestCase):
+    uses_mmap = False
+
     from mutagen.id3 import TCON
     from mutagen._constants import GENRES
 
@@ -865,6 +1157,8 @@ class Genres(TestCase):
         self.assertEquals(gen.genres, [u"Unknown", u"genre"])
 
 class BrokenDiscarded(TestCase):
+    uses_mmap = False
+
     def test_empty(self):
         from mutagen.id3 import TPE1, ID3JunkFrameError
         self.assertRaises(ID3JunkFrameError, TPE1.fromData, _24, 0x00, '')
@@ -880,7 +1174,8 @@ class BrokenDiscarded(TestCase):
         tail = '\x00\x00\x00\x03\x00\x00' '\x01\x02\x03'
         for head in 'RVA2 TXXX APIC'.split():
             data = head + tail
-            self.assertEquals(0, len(list(id3.read_frames(data, Frames))))
+            self.assertEquals(
+                0, len(list(id3._ID3__read_frames(data, Frames))))
 
     def test_drops_nonalphanum_frames(self):
         from mutagen.id3 import Frames
@@ -888,7 +1183,8 @@ class BrokenDiscarded(TestCase):
         tail = '\x00\x00\x00\x03\x00\x00' '\x01\x02\x03'
         for head in ['\x06\xaf\xfe\x20', 'ABC\x00', 'A   ']:
             data = head + tail
-            self.assertEquals(0, len(list(id3.read_frames(data, Frames))))
+            self.assertEquals(
+                0, len(list(id3._ID3__read_frames(data, Frames))))
 
     def test_bad_unicodedecode(self):
         from mutagen.id3 import COMM, ID3JunkFrameError
@@ -897,6 +1193,8 @@ class BrokenDiscarded(TestCase):
         self.assertRaises(ID3JunkFrameError, COMM.fromData, _24, 0x00, data)
 
 class BrokenButParsed(TestCase):
+    uses_mmap = False
+
     def test_missing_encoding(self):
         from mutagen.id3 import TIT2
         tag = TIT2.fromData(_23, 0x00, 'a test')
@@ -911,7 +1209,8 @@ class BrokenButParsed(TestCase):
         tail = '\x00' * 6
         for head in 'WOAR TENC TCOP TOPE WXXX'.split():
             data = head + tail
-            self.assertEquals(0, len(list(id3.read_frames(data, Frames))))
+            self.assertEquals(
+                0, len(list(id3._ID3__read_frames(data, Frames))))
 
     def test_lengthone_utf16(self):
         from mutagen.id3 import TPE1
@@ -924,7 +1223,8 @@ class BrokenButParsed(TestCase):
         from mutagen.id3 import TPE1, Frame, ID3BadCompressedData
         id3 = ID3()
         id3.PEDANTIC = True
-        self.assertRaises(ID3BadCompressedData, TPE1.fromData, id3, Frame.FLAG24_COMPRESS, '\x03abcdefg')
+        self.assertRaises(ID3BadCompressedData, TPE1.fromData, id3,
+                          Frame.FLAG24_COMPRESS, '\x03abcdefg')
 
     def test_fake_zlib_nopedantic(self):
         from mutagen.id3 import TPE1, Frame, ID3BadCompressedData
@@ -940,13 +1240,20 @@ class BrokenButParsed(TestCase):
         self.assertEquals(tag.encoding, 1)
         self.assertEquals(tag, ['this is a/test'])
 
+    def test_zlib_latin1_missing_datalen(self):
+        from mutagen.id3 import TPE1
+        tag = TPE1.fromData(_24, 0x8, '\x00\x00\x00\x0f'
+                'x\x9cc(\xc9\xc8,V\x00\xa2D\xfd\x92\xd4\xe2\x12\x00&\x7f\x05%')
+        self.assertEquals(tag.encoding, 0)
+        self.assertEquals(tag, ['this is a/test'])
+
     def test_detect_23_ints_in_24_frames(self):
         from mutagen.id3 import Frames
         head = 'TIT1\x00\x00\x01\x00\x00\x00\x00'
         tail = 'TPE1\x00\x00\x00\x04\x00\x00Yay!'
 
-        tagsgood = list(_24.read_frames(head + 'a'*127 + tail, Frames))
-        tagsbad = list(_24.read_frames(head + 'a'*255 + tail, Frames))
+        tagsgood = list(_24._ID3__read_frames(head + 'a'*127 + tail, Frames))
+        tagsbad = list(_24._ID3__read_frames(head + 'a'*255 + tail, Frames))
         self.assertEquals(2, len(tagsgood))
         self.assertEquals(2, len(tagsbad))
         self.assertEquals('a'*127, tagsgood[0])
@@ -954,8 +1261,8 @@ class BrokenButParsed(TestCase):
         self.assertEquals('Yay!', tagsgood[1])
         self.assertEquals('Yay!', tagsbad[1])
 
-        tagsgood = list(_24.read_frames(head + 'a'*127, Frames))
-        tagsbad = list(_24.read_frames(head + 'a'*255, Frames))
+        tagsgood = list(_24._ID3__read_frames(head + 'a'*127, Frames))
+        tagsbad = list(_24._ID3__read_frames(head + 'a'*255, Frames))
         self.assertEquals(1, len(tagsgood))
         self.assertEquals(1, len(tagsbad))
         self.assertEquals('a'*127, tagsgood[0])
@@ -963,6 +1270,8 @@ class BrokenButParsed(TestCase):
 
 
 class TimeStamp(TestCase):
+    uses_mmap = False
+
     from mutagen.id3 import ID3TimeStamp as Stamp
 
     def test_Y(self):
@@ -1035,7 +1344,7 @@ class OddWrites(TestCase):
 
     def test_toemptyfile(self):
         os.unlink(self.newsilence)
-        file(self.newsilence, "w").close()
+        file(self.newsilence, "wb").close()
         ID3(self.silence).save(self.newsilence)
 
     def test_tononfile(self):
@@ -1044,12 +1353,12 @@ class OddWrites(TestCase):
 
     def test_1bfile(self):
         os.unlink(self.newsilence)
-        f = file(self.newsilence, "w")
+        f = file(self.newsilence, "wb")
         f.write("!")
         f.close()
         ID3(self.silence).save(self.newsilence)
         self.assert_(os.path.getsize(self.newsilence) > 1)
-        self.assertEquals(file(self.newsilence).read()[-1], "!")
+        self.assertEquals(file(self.newsilence, "rb").read()[-1], "!")
 
     def tearDown(self):
         try: os.unlink(self.newsilence)
@@ -1096,17 +1405,19 @@ class WriteRoundtrip(TestCase):
         id3 = ID3(self.newsilence)
         self.assertEquals(id3["TPE1"], ["jzig", "piman"])
 
+    def test_compressibly_large(self):
+        from mutagen.id3 import TPE2
+        f = ID3(self.newsilence)
+        self.assert_("TPE2" not in f)
+        f["TPE2"] = TPE2(encoding=0, text="Ab" * 1025)
+        f.save()
+        id3 = ID3(self.newsilence)
+        self.assertEquals(id3["TPE2"], "Ab" * 1025)
+
     def test_nofile_emptytag(self):
         os.unlink(self.newsilence)
         ID3().save(self.newsilence)
-        self.assertEquals('ID3', open(self.newsilence).read(3))
-        self.assertEquals(ID3(self.newsilence), {})
-
-    def test_emptyfile_emptytag(self):
-        open(self.newsilence, 'wb').truncate()
-        ID3().save(self.newsilence)
-        self.assertEquals('ID3', open(self.newsilence).read(3))
-        self.assertEquals(ID3(self.newsilence), {})
+        self.assertRaises(EnvironmentError, open, self.newsilence)
 
     def test_nofile_silencetag(self):
         id3 = ID3(self.newsilence)
@@ -1124,14 +1435,28 @@ class WriteRoundtrip(TestCase):
 
     def test_empty_plustag_minustag_empty(self):
         id3 = ID3(self.newsilence)
-        #os.unlink(self.newsilence)
         open(self.newsilence, 'wb').truncate()
-        id3.save(self.newsilence)
-        id3.delete(self.newsilence)
+        id3.save()
+        id3.delete()
+        self.failIf(id3)
         self.assertEquals(open(self.newsilence).read(10), '')
 
+    def test_empty_plustag_emptytag_empty(self):
+        id3 = ID3(self.newsilence)
+        open(self.newsilence, 'wb').truncate()
+        id3.save()
+        id3.clear()
+        id3.save()
+        self.assertEquals(open(self.newsilence).read(10), '')
+
+    def test_delete_invalid_zero(self):
+        open(self.newsilence, 'wb').write('ID3\x04\x00\x00\x00\x00\x00\x00abc')
+        ID3(self.newsilence).delete()
+        self.assertEquals(open(self.newsilence).read(10), 'abc')
+
     def tearDown(self):
-        os.unlink(self.newsilence)
+        try: os.unlink(self.newsilence)
+        except EnvironmentError: pass
 
 class WriteForEyeD3(TestCase):
     silence = join('tests', 'data', 'silence-44-s.mp3')
@@ -1179,145 +1504,47 @@ class WriteForEyeD3(TestCase):
     def tearDown(self):
         os.unlink(self.newsilence)
 
-class FileHandling(TestCase):
-    def setUp(self):
-        self.id3 = ID3()
+class NoHash(TestCase):
+    uses_mmap = False
 
-    def tearDown(self):
-        del self.id3
+    def test_spec(self):
+        from mutagen.id3 import Spec
+        self.failUnlessRaises(TypeError, {}.__setitem__, Spec("foo"), None)
 
-    def file(self, contents):
-        import tempfile
-        temp = tempfile.TemporaryFile()
-        temp.write(contents)
-        temp.flush()
-        temp.seek(0)
-        return temp
+    def test_frame(self):
+        from mutagen.id3 import TIT1
+        self.failUnlessRaises(
+            TypeError, {}.__setitem__, TIT1(encoding=0, text="foo"), None)
 
-    def read(self, fobj):
-        fobj.seek(0, 0)
-        return fobj.read()
+class FrameIDValidate(TestCase):
+    uses_mmap = False
 
-    def test_insert_into_empty(self):
-        o = self.file('')
-        self.id3._insert_space(o, 8, 0)
-        self.assertEquals('\x00' * 8, self.read(o))
+    def test_valid(self):
+        from mutagen.id3 import is_valid_frame_id
+        self.failUnless(is_valid_frame_id("APIC"))
+        self.failUnless(is_valid_frame_id("TPE2"))
 
-    def test_insert_before_one(self):
-        o = self.file('a')
-        self.id3._insert_space(o, 8, 0)
-        self.assertEquals('a' + '\x00' * 7 + 'a', self.read(o))
+    def test_invalid(self):
+        from mutagen.id3 import is_valid_frame_id
+        self.failIf(is_valid_frame_id("MP3e"))
+        self.failIf(is_valid_frame_id("+ABC"))
 
-    def test_insert_after_one(self):
-        o = self.file('a')
-        self.id3._insert_space(o, 8, 1)
-        self.assertEquals('a' + '\x00' * 8, self.read(o))
-
-    def test_smaller_than_file_middle(self):
-        o = self.file('abcdefghij')
-        self.id3._insert_space(o, 4, 4)
-        self.assertEquals('abcdefghefghij', self.read(o))
-
-    def test_smaller_than_file_to_end(self):
-        o = self.file('abcdefghij')
-        self.id3._insert_space(o, 4, 6)
-        self.assertEquals('abcdefghijghij', self.read(o))
-
-    def test_smaller_than_file_across_end(self):
-        o = self.file('abcdefghij')
-        self.id3._insert_space(o, 4, 8)
-        self.assertEquals('abcdefghij\x00\x00ij', self.read(o))
-
-    def test_smaller_than_file_at_end(self):
-        o = self.file('abcdefghij')
-        self.id3._insert_space(o, 3, 10)
-        self.assertEquals('abcdefghij\x00\x00\x00', self.read(o))
-
-    def test_smaller_than_file_at_beginning(self):
-        o = self.file('abcdefghij')
-        self.id3._insert_space(o, 3, 0)
-        self.assertEquals('abcabcdefghij', self.read(o))
-
-    def test_zero(self):
-        o = self.file('abcdefghij')
-        self.assertRaises((AssertionError, ValueError), self.id3._insert_space, o, 0, 1)
-
-    def test_negative(self):
-        o = self.file('abcdefghij')
-        self.assertRaises((AssertionError, ValueError), self.id3._insert_space, o, 8, -1)
-
-    def test_delete_one(self):
-        o = self.file('a')
-        self.id3._delete_bytes(o, 1, 0)
-        self.assertEquals('', self.read(o))
-
-    def test_delete_first_of_two(self):
-        o = self.file('ab')
-        self.id3._delete_bytes(o, 1, 0)
-        self.assertEquals('b', self.read(o))
-
-    def test_delete_second_of_two(self):
-        o = self.file('ab')
-        self.id3._delete_bytes(o, 1, 1)
-        self.assertEquals('a', self.read(o))
-
-    def test_delete_third_of_two(self):
-        o = self.file('ab')
-        self.assertRaises(AssertionError, self.id3._delete_bytes, o, 1, 2)
-
-    def test_delete_middle(self):
-        o = self.file('abcdefg')
-        self.id3._delete_bytes(o, 3, 2)
-        self.assertEquals('abfg', self.read(o))
-
-    def test_delete_across_end(self):
-        o = self.file('abcdefg')
-        self.assertRaises(AssertionError, self.id3._delete_bytes, o, 4, 8)
-
-    def test_delete_zero(self):
-        o = self.file('abcdefg')
-        self.assertRaises(AssertionError, self.id3._delete_bytes, o, 0, 3)
-
-    def test_delete_negative(self):
-        o = self.file('abcdefg')
-        self.assertRaises(AssertionError, self.id3._delete_bytes, o, 4, -8)
-
-class FileHandlingNoMMap(FileHandling):
-    # run the FileHandling tests with a broken mmap#move to simulate amd64
-    class MockMMap(object):
-        def __init__(self, *args, **kwargs): pass
-        def move(self, dest, src, count): raise ValueError
-        def close(self): pass
-
-    from mmap import mmap as __real_mmap_mmap
-
-    def setUp(self):
-        super(FileHandlingNoMMap, self).setUp()
-        import mmap
-        mmap.mmap = self.MockMMap
-
-    def tearDown(self):
-        super(FileHandlingNoMMap, self).tearDown()
-        import mmap
-        mmap.mmap = self.__real_mmap_mmap
-
-
-registerCase(ID3Loading)
-registerCase(ID3GetSetDel)
-registerCase(BitPaddedIntTest)
-registerCase(ID3Tags)
-registerCase(ID3v1Tags)
-registerCase(BrokenDiscarded)
-registerCase(BrokenButParsed)
-registerCase(FrameSanityChecks)
-registerCase(SpecSanityChecks)
-registerCase(Genres)
-registerCase(TimeStamp)
-registerCase(WriteRoundtrip)
-registerCase(OddWrites)
-registerCase(FileHandling)
-registerCase(FileHandlingNoMMap)
+add(ID3Loading)
+add(ID3GetSetDel)
+add(BitPaddedIntTest)
+add(ID3Tags)
+add(ID3v1Tags)
+add(BrokenDiscarded)
+add(BrokenButParsed)
+add(FrameSanityChecks)
+add(SpecSanityChecks)
+add(Genres)
+add(TimeStamp)
+add(WriteRoundtrip)
+add(OddWrites)
+add(NoHash)
+add(FrameIDValidate)
 
 try: import eyeD3
 except ImportError: pass
-else: registerCase(WriteForEyeD3)
+else: add(WriteForEyeD3)
